# EasyMaker.SDK

* AI EasyMaker 서비스의 Python SDK를 위한 저장소이다.
* 학습, 모델 등의 리소스를 생성하고 관리할 수 있는 기능을 제공한다.
* [SDK 사용 가이드 - NHN Cloud 사용자 가이드](https://docs.nhncloud.com/ko/Machine%20Learning/AI%20EasyMaker/ko/sdk-guide/)
* SDK를 사용하여 AI EasyMaker를 사용하는 예제는 docs 디렉터리 하위에 Juptyer Notebook으로 제공한다.

## 프로젝트 구조

> {prjoectRoot}\
> ├── ./src/easymaker : 소스 코드\
> ├── ./tests : 테스트 코드\
> ├── ./docs : 사용 참고 코드.\
> ├── ./pyproject.toml : 프로젝트 설정 파일\
> ├── ./docs/README.md : PyPI에 올라갈 README 파일\
> └── ./README.md : 내부 README 파일

### ./src/easymaker 디렉터리 주요 정보

* `initializer.py`: 기본 정보 셋팅 (앱키, 리전 등)
* `api`: AI EasyMaker API를 호출하기 위한 클라이언트 코드
* `cli`: Instance, Image 정보 조회 등을 위한 CLI 명령 지원
* `storage`: Object Storage 서비스의 오브젝트 업로드 및 다운로드 기능 지원
* `experiment`: 실험 기능
* `training`: 학습, 하이퍼파라미터 튜닝 기능
* `model`: 모델 기능
* `endpoint`: 엔드포인트 기능
* `batch_inference`: 배치 추론 기능
* `pipeline`: ML 파이프라인 기능
* `log`: Log & Crash Search 서비스의 로그 수집 기능 지원

## 설치

> `uv sync`로 설치되는 의존성에는 개발 환경에 필요한 패키지도 포함된다.

### 파이썬 설치

```sh
uv python install 3.10
```

### 현재 소스의 패키지 설치

```sh
uv sync
```

### PyPI에 등록된 패키지 설치

```sh
uv sync --no-install-project
uv pip install easymaker=={버전}
```

## uv 명령어

> 명령어의 자세한 설명이나 다른 명령어를 확인하려면 `uv {명령어}` 또는 `uv --help`를 입력한다.

### python 실행

```sh
uv run python
```

### `pytest` 실행

```sh
uv run pytest -s -v
```

### `ruff` 포맷터 실행

```sh
uv run ruff check
uv run ruff check --fix
uv run ruff check --fix --unsafe-fixes
```

## CLI 실행

```sh
easymaker [options]
```

위와 같이 실행시 cli.py가 실행됨, 여기에 CLI 호출 코드 추가

## 개발 환경 세팅

* 코드 포맷터
    * Python 포맷터는 ruff를 사용한다.
    * 설정 값은 pyproject.toml에 정의되어 있다.
* IntelliJ, PyCharm # TODO: Intellij, PyCharm 설정 수정
    * 코드 포맷은 Actions on Save를 사용한다.
    * isort는 별도 설정으로 실행하여 적용한다.
        * 단축키로 설정하여 사용 가능.
    * 기타 설정
        * Editor > General > Auto Import: Python 섹션의 Show auto-import tooltip을 체크한다.
    * Black 설정 방법
        * Settings > Plugins: Python 플러그인을 설치한다.
        * Settings > Tools > Actions on Save:
            * Reformat code, Optimize Imports에서 Python 파일을 제외한다.
            * Run Black을 체크한다.
        * Settings > Tools > Black:
            * On code reformat, On save를 체크한다.
            * Settings에 `--line-length 500`를 입력한다.
    * isort 설정 방법
        * 파이썬 가상환경에 isort를 설치한다.
        * Settings > Tools > External Tools > __[+]__ 클릭하여 추가
            * Name, Group: isort
            * Description: Sort imports in python file
            * Program: {프로젝트 루트}/venv/bin/isort
            * Arguments: --profile black
            * Working directory: {프로젝트 루트}
            * Advanced Options: 전부 체크
        * Settings > Keymap:
            * External Tools > isort에 단축키를 `Ctrl` + `Option` + `O` 로 설정
* VSCode
    * 프로젝트 단위 설정이 .vscode에 저장되어 있다.
    * 포맷터 적용을 위해 vscode의 확장 프로그램 탭에서 추천 확장 프로그램을 설치한다.
        * 추천 확장 프로그램은 .vscode/extensions.json에 정의되어 있다.
